package cz.muni.fi.mir.mathmlcanonicalization.modules;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;
import org.jdom2.Document;
import org.jdom2.Element;

import cz.muni.fi.mir.mathmlcanonicalization.Settings;

// *** MODULE REPLACED BY OperatorNormalizer ***

/**
 * Normalize the way to express an function applied to arguments in MathML.
 * <h4>Input</h4> Well-formed MathML, not processed by MrowMinimizer yet
 * <h4>Output</h4> The original code with:<ul> <li>removed entities for function
 * application (and multiplying where it should not be)</li> <li>the name of
 * function placed in &lt;mi&gt; element (not &lt;mo&gt;)</li> <li>function
 * arguments placed in parentheses and &lt;mrow&gt; (or leave &lt;mrow&gt;
 * adding for the MrowNormalizer module?)</li></ul>
 *
 * @author David Formanek
 */
public class FunctionNormalizer {

    final static String applyFunction = Character.valueOf((char) Integer.parseInt("02061", 16)).toString();

    /**
     * Normalize the way to express an function applied to arguments.
     *
     * @param doc DOM document to be canonicalized
     */
    public static void execute(Document doc) {        
        normalizeFunction(doc.getRootElement());
    }    
    
    private static boolean normalizeFunction(Element element) {
        if (element.getName().equals("mo")) {
            if (element.getText().equals(applyFunction)) {
                return true;
            }
        }
        Iterator<Element> iterator = element.getChildren().iterator();
        while(iterator.hasNext()) {
            if (normalizeFunction(iterator.next())) {
                iterator.remove();
            }
        }
        return false;
    }

    // FunctionNormalizer is a JDOM module for now
    
    public static ByteArrayOutputStream execute(final InputStream input) {
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            XMLInputFactory inputFactory = Settings.setupXMLInputFactory();
            XMLOutputFactory outputFactory = XMLOutputFactory.newInstance();
            // stream for reading event from input stream
            XMLStreamReader reader = inputFactory.createXMLStreamReader(input);
            // stream that writes events to given output stream
            final XMLStreamWriter writer = outputFactory.createXMLStreamWriter(outputStream, "UTF-8");
            writer.writeStartDocument(reader.getEncoding(), reader.getVersion());
            // check for event
            boolean mo = false;
            boolean write = true;
            Map<QName, String> attrs = new HashMap<QName, String>();
            QName qName = null;
            while (reader.hasNext()) {
                // get event code
                final int event = reader.next();
                // based on event code choose action
                switch (event) {
                    case XMLStreamConstants.START_ELEMENT: {
                        attrs.clear();
                        qName = reader.getName();
                        for (int index = 0; index < reader.getAttributeCount(); ++index) {
                            String attributeValue = reader.getAttributeValue(index);
                            QName attrName = reader.getAttributeName(index);
                            attrs.put(attrName, attributeValue);
                            
                        }
                        
                        if (qName.getLocalPart().equals("mo")) {
                            mo = true;
                        } else {
                            mo = false;
                            writer.writeStartElement(qName.getPrefix(), qName.getLocalPart(), qName.getNamespaceURI());
                            for (int index = 0; index < reader.getNamespaceCount(); ++index) {
                                writer.writeNamespace(reader.getNamespacePrefix(index), reader.getNamespaceURI(index));
                            }
                            for (Map.Entry<QName, String> attr : attrs.entrySet()) {
                                writeAttribute(attr.getKey(), attr.getValue(), writer);
                            }
                        }
                        break;
                    }
                    case XMLStreamConstants.END_ELEMENT: {
                        if (write) {
                            writer.writeEndElement();
                        }
                        write = true;
                        qName = null;
                        mo = false;
                        break;
                    }
                    case XMLStreamConstants.CHARACTERS: {
                        String text = reader.getText();
                        if (mo) {
                            if (!text.equals(applyFunction)) {
                                write = true;
                                writer.writeStartElement(qName.getPrefix(), qName.getLocalPart(), qName.getNamespaceURI());
                                for (Map.Entry<QName, String> attr : attrs.entrySet()) {
                                    writeAttribute(attr.getKey(), attr.getValue(), writer);
                                }
                            } else {
                                write = false;
                            }
                        }
                        if (write) {
                            writer.writeCharacters(reader.getText());
                        }
                        break;
                    }
                    case XMLStreamConstants.ENTITY_REFERENCE: {
                        break;
                    }
                    case XMLStreamConstants.END_DOCUMENT: {
                        writer.writeEndDocument();
                        break;
                    }
                    case XMLStreamConstants.COMMENT: {
                        break;
                    }
                    case XMLStreamConstants.DTD: {
                        writer.writeDTD(reader.getText());
                    }
                    default: {
                        break;
                    }
                }
            }
            writer.flush();
            writer.close();
        } catch (final XMLStreamException ex) {
            ex.printStackTrace();
        }
        return outputStream;
    }

    private static void writeAttribute(QName qName, String value, XMLStreamWriter writer) throws XMLStreamException {
        if (qName.getNamespaceURI() == null) {
            writer.writeAttribute(qName.getLocalPart(), value);
        } else {
            writer.writeAttribute(qName.getPrefix(), qName.getNamespaceURI(), qName.getLocalPart(), value);
        }
    }
    
}
